'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

exports.default = function (_ref) {
  var apolloClient = _ref.apolloClient;


  function observableQueryFields(observable) {

    var fields = {};

    ['variables', 'refetch', 'fetchMore', 'updateQuery', 'startPolling', 'stopPolling', 'subscribeToMore'].forEach(function (key) {
      if (typeof observable[key] === 'function') {
        fields[key] = observable[key].bind(observable);
      } else {
        fields[key] = observable[key];
      }
    });

    return fields;
  }

  return function (document, options) {

    var operation = parser(document);

    var GraphQL = function () {
      function GraphQL(_ref2) {
        var _ref2$options = _ref2.options,
            options = _ref2$options === undefined ? {} : _ref2$options;

        _classCallCheck(this, GraphQL);

        this.queryObservable = null;
        this.querySubscription = null;
        this.initialOptions = options;
        this.computedOptions = typeof options === "string";
        this.initialLoadingDone = false;
        this.rid = Math.floor(1000000000 + Math.random() * 9000000000);
      }

      _createClass(GraphQL, [{
        key: 'init',
        value: function init(el) {
          var _this = this;

          if (this.computedOptions) {
            (function () {
              var self = _this;
              el['__apollo_' + _this.rid] = function (change) {
                self.optionsChanged(this, change);
              };
              el._addComplexObserverEffect('__apollo_' + _this.rid + '(' + _this.initialOptions + '.*)');
            })();
          } else {
            // static options
            this.currentOptions = this.initialOptions;
          }

          if (operation.type === "Mutation") {
            this.setDataToProp(el);
            return;
          }
        }
      }, {
        key: 'optionsChanged',
        value: function optionsChanged(el, change) {
          if (change.path === this.initialOptions) {
            //console.log("full change");
          }

          if (change.base.skip) {
            if (this.currentOptions && !this.currentOptions.skip) {
              return this.unsubscribeFromQuery();
            }
            return; // skip an already skipped
          }

          this.currentOptions = change.base;
          this.subscribeToQuery(el);
        }
      }, {
        key: 'ready',
        value: function ready(el) {
          if (operation.type === "Mutation" || this.querySubscription) {
            return;
          }

          if (!this.computedOptions) {
            this.subscribeToQuery(el);
          }
        }
      }, {
        key: 'generateQueryOptions',
        value: function generateQueryOptions() {
          return (0, _lodash2.default)(this.currentOptions, ['skip', 'name']);
        }
      }, {
        key: 'createQuery',
        value: function createQuery() {
          if (operation.type === "Query") {
            this.queryObservable = apolloClient.watchQuery(Object.assign({
              query: document
            }, this.generateQueryOptions()));
          }
        }
      }, {
        key: 'subscribeToQuery',
        value: function subscribeToQuery(el) {
          var _this2 = this;

          if (operation.type === "Mutation") {
            return;
          }

          if (this.querySubscription) {
            this.queryObservable.setOptions(this.generateQueryOptions());
            return;
          }

          if (!this.queryObservable) {
            this.createQuery();
          } else {
            // unskipped query, update options.
            this.queryObservable.setOptions(this.generateQueryOptions());
          }

          var next = function next(result) {
            _this2.setDataToProp(el, result);
          };

          var handleError = function handleError(error) {
            console.log(error);
            //if (error instanceof ApolloError) return next({ error });
            //throw error;
          };

          this.querySubscription = this.queryObservable.subscribe({
            next: next,
            error: handleError
          });
        }
      }, {
        key: 'setDataToProp',
        value: function setDataToProp(el, result) {
          var name = operation.type === "Mutation" ? "mutate" : "data";
          if (this.currentOptions && this.currentOptions.name) {
            name = this.currentOptions.name;
          }

          if (operation.type === "Mutation") {
            return el.set(name, function (opts) {
              opts.mutation = document;
              return apolloClient.mutate(opts);
            });
          }

          var data = result.data,
              loading = result.loading,
              networkStatus = result.networkStatus;


          var elmData = Object.assign({}, data, {
            loading: loading,
            networkStatus: networkStatus
          }, observableQueryFields(this.queryObservable));

          el.set(name, elmData);
        }
      }, {
        key: 'unsubscribeFromQuery',
        value: function unsubscribeFromQuery() {
          if (this.querySubscription) {
            this.querySubscription.unsubscribe();
            delete this.querySubscription;
          }
        }
      }]);

      return GraphQL;
    }();

    var graphql = new GraphQL({ options: options });

    return {
      beforeRegister: function beforeRegister() {
        graphql.init(this);
      },


      registered: function registered() {},

      ready: function ready() {
        graphql.ready(this);
      },
      attached: function attached() {},
      detached: function detached() {
        //graphql.detached(this);
      }
    };
  };
};

var _lodash = require('lodash.omit');

var _lodash2 = _interopRequireDefault(_lodash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function parser(document) {
  // variables
  var fragments = void 0,
      queries = void 0,
      mutations = void 0,
      subscriptions = void 0,
      variables = void 0,
      definitions = void 0,
      type = void 0,
      name = void 0;

  if (!document && !document.kind) {
    // tslint:disable-line
    throw new Error('Argument of ' + document + ' passed to parser was not a valid GraphQL Document. You may need to use \'graphql-tag\' or another method to convert your operation into a document');
  }

  fragments = document.definitions.filter(function (x) {
    return x.kind === 'FragmentDefinition';
  });

  /*fragments = createFragment({
    kind: 'Document',
    definitions: [...fragments],
  });*/

  queries = document.definitions.filter(function (x) {
    return x.kind === 'OperationDefinition' && x.operation === 'query';
  });

  mutations = document.definitions.filter(function (x) {
    return x.kind === 'OperationDefinition' && x.operation === 'mutation';
  });

  subscriptions = document.definitions.filter(function (x) {
    return x.kind === 'OperationDefinition' && x.operation === 'subscription';
  });

  if (fragments.length && (!queries.length || !mutations.length || !subscriptions.length)) {
    throw new Error('Passing only a fragment to \'graphql\' is not yet supported. You must include a query, subscription or mutation as well');
  }

  if (queries.length && mutations.length && mutations.length) {
    if (queries.length + mutations.length + mutations.length > 1) {
      throw new Error('poly-apollo only supports a query, subscription, or a mutation per Behavior. ' + document + ' had ' + queries.length + ' queries, ' + subscriptions.length + ' subscriptions and ' + mutations.length + ' muations. You can use \'compose\' to join multiple operation types to a component');
    }
  }

  type = queries.length ? "Query" : "Mutation";
  if (!queries.length && !mutations.length) type = "Subscription";

  definitions = queries.length ? queries : mutations;
  if (!queries.length && !mutations.length) definitions = subscriptions;

  if (definitions.length !== 1) {
    throw new Error('poly-apollo only supports one defintion per HOC. ' + document + ' had ' + definitions.length + ' definitions. You can use \'compose\' to join multiple operation types to a component');
  }

  variables = definitions[0].variableDefinitions || [];
  var hasName = definitions[0].name && definitions[0].name.kind === 'Name';
  name = hasName ? definitions[0].name.value : 'data'; // fallback to using data if no name
  fragments = fragments.length ? fragments : [];

  return { name: name, type: type, variables: variables, fragments: fragments };
}

;